package fr.unice.javinspector.GUI.main;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * AnimatedPanel is a JPanel containing a animation of images gif painted as icon. <br>
 * You can use this class in order to animate a component with images not to big. <br>
 * The system of animation will put all the gif images in a array and a timer will throw 
 * a action event to display them. By default the frequence is 50 ms.
 * <br> The images have to be in a special syntax : NAMEi.gi [ ampoule0.gif
 *  or ampoule8.gif for example] with i begining at 0 and finishing at the last 
 * index of images you want to animate in a successive order.
 * <br>A cycle is a animation of images wich ca be compared to a sinusoid. [ for example
 * you have 2 images . in the array we will create an array of 4 images and array[0] = im0.gif ,
 * array[1]=im1.gif, array[2]=im1.gif and array[3]=im0.gif ].
 * @version final 09/03/04
 * @author ARNAUD J-M
 */
public class AnimatedPanel extends JPanel
                          implements ActionListener {
   /** the array of ImageIcon */                    	
   private ImageIcon images[];
   /** the number of images to animate */
   private int nbre;
   /** the index of the current image displayed */
   private int current = 0;
   /** the delay of the timer for the animation*/
   private int delai;
   /** the minimum height of the panel */
   int hauteur;
   /*  the minimum width of the panel */
   private int largeur;
   /** the timer which manages the animation*/
   private Timer animateur;

/** 
 * This constructor creates a AnimatedPanel that contains a cycle with frequence of 50 ms.
 * @param nom the String of the base name of images gif.
 * @param nombreMax the number of images we want to animate.
 * @param larg the minimum width of the panel.
 * @param haut the minimum height of the panel.
 */
public AnimatedPanel(String nom , int nombreMax , int larg , int haut){
	this(nom,nombreMax,larg,haut,true,50);
}

/** 
 * This constructor creates a AnimatedPanel.
 * @param nom the String of the base name of images gif.
 * @param nombreMax the number of images we want to animate.
 * @param larg the minimum width of the panel.
 * @param haut the minimum height of the panel.
 * @param cycle the boolean indicates if the animation is a cycle.
 * @param del the frequence of animation.
 */
   public AnimatedPanel(String nom , int nombreMax ,int larg,int haut,boolean cycle,int del){
   	delai = del;
   	largeur = larg;
   	hauteur = haut;
   	nbre = cycle?2*nombreMax:nombreMax;
    images = new ImageIcon[nbre];
    // we initiliazise the array of images
      for ( int i = 0; i < nombreMax; ++i ) 
         images[i] = new ImageIcon(AnimatedPanel.class.getResource("/"+nom+i+".gif"));
       if(cycle){
       //	if the animation is a cycle we reverse the images at the end of the array
       	 int j = nombreMax ;
        for(int i=nombreMax; i< 2 * nombreMax ;i++){
         j--;
         images[i]=new ImageIcon(AnimatedPanel.class.getResource("/"+nom+j+".gif"));
         }
        }
      startAnimation();
   }
   
   /**
    * paintComponent calls PaintComponent of the mother class JPanel, and paints
    * the current image verifying it is completely loaded.
    */
	   public void paintComponent(Graphics g){
      super.paintComponent(g);
      if ( images[current].getImageLoadStatus() ==
           MediaTracker.COMPLETE ) 
         images[current].paintIcon( this, g, 0, 0 );
   }

/** 
 * This procedure will be called at each bip of timer.
 * It increased the index of current image if it was completely loaded
 * after repainted the panel.
 */
   public void actionPerformed(ActionEvent e){
   	if ( images[current].getImageLoadStatus() ==
          MediaTracker.COMPLETE ) {
         repaint();
         current = ( current + 1 ) % nbre;
         }
   }

/** 
 * startAnimation launches the animation.
 **/
   public void startAnimation(){
   	if (animateur == null){
         animateur = new Timer(delai,this);
         animateur.start();
         }
    else if (! animateur.isRunning())
    		animateur.restart();
   }

/**
 * stopAnimation stops the animation stopping the timer.
 */
   public void stopAnimation(){
      animateur.stop();
   }
   
   /**
    * to get the minimum size of the animatedPanel.
    * @return the minimum Dimension of this panel.
    */
    public Dimension getMinimumSize(){ 
      return getPreferredSize(); 
   }

	/**
    * to get the prefered size of the animatedPanel.
    * @return the preferred Dimension of this panel.
    */
   public Dimension getPreferredSize(){
      return new Dimension(largeur,hauteur);
   }
}
