package fr.unice.javinspector.GUI.search;

import java.util.ArrayList;
import javax.swing.JPanel;
import fr.unice.javinspector.GUI.util.VerticalFlowLayout;
import java.awt.Color;
import javax.swing.JLabel;
import fr.unice.javinspector.core.search.SearchException;
import java.util.Iterator;
import java.lang.reflect.Member.*;
import java.lang.reflect.Member;
import java.io.File;
import javax.swing.ImageIcon;
import java.net.URL;
import java.awt.Image;
import java.awt.Toolkit;

/**
 * <p>Title: FrontEndResultSearch</p>
 * <p>Description: This class extends the JPanel. This is generating a formated
 * text in a JPanel with a result of a search.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: UNICE</p>
 * @author Gahide jrme - Zerbib Gabriel - Galbois Xavier - Charbit Lucas
 * @version 1.0
 */

public class FrontEndResultSearch extends JPanel{

  private VerticalFlowLayout verticalFlowLayout = new VerticalFlowLayout();
  private JLabel jl_title;

  /**
   * Construct a new result for a search in a JPanel
   * @param searchResultList an ArrayList of Member
   * @param title the title of the result
   * @throws SearchException if searchResultList is null
   * @see JPanel
   * @see Member
   */
  public FrontEndResultSearch(ArrayList searchResultList, String title)
      throws SearchException {
    super();
    //exception managment
    if( searchResultList == null)
      throw new SearchException(SearchException.INTERNAL_ERROR_NULL_POINTER);

    this.setLayout(verticalFlowLayout);
    this.setBackground(Color.BLUE);
    jl_title = new JLabel();

    if( searchResultList.isEmpty()){
            putTitle("No result found for "+title);
    }
    else{
      putTitle("Search result for "+title);
      Iterator iter = searchResultList.iterator();
      while (iter.hasNext()) {
        Member item = (Member)iter.next();
        addEntries(item.toString(),item);
      }
    }




  }

  /**
   * Create an empty result FrontEndResultSearch.
   * It can be used for the result search front end that has to be init
   * but no search has been done yet.
   */
  public FrontEndResultSearch(){
    super();
    this.setBackground(Color.RED);
    jl_title = new JLabel();
    jl_title.setFont(new java.awt.Font("Dialog",0,24));
    jl_title.setText("No Result search, cause no search ");
    this.add(jl_title);
  }


  // private methods for fromatting text in a JPANEL


  private void putTitle(String title){
    jl_title.setFont(new java.awt.Font("Dialog",0,24));
    jl_title.setText(title);
//    jl_title.setBackground(Color.RED);
    this.add(jl_title);

  }


  private void putSection(String section){
    JLabel jl_section = new JLabel("\t"+section);
    jl_section.setFont(new java.awt.Font("Dialog",3,14));
    this.add(jl_section);
  }

  private void addEntries(String resultEntries, Member item){
    String sep = File.separator;
    JLabel jl_entries = new JLabel("\t\t"+resultEntries);
    jl_entries.setFont(new java.awt.Font("Dialog",3,14));

    if(item instanceof java.lang.reflect.Method) {
      URL url = FrontEndResultSearch.class.getResource("/imgs/method.gif");
      jl_entries.setIcon(new ImageIcon(url));
//    jl_entries.setIcon(new ImageIcon("imgs" + sep + "method.gif"));
    }
    if(item instanceof java.lang.reflect.Field) {
      URL url = FrontEndResultSearch.class.getResource("/imgs/variable.gif");
      jl_entries.setIcon(new ImageIcon(url));
//      jl_entries.setIcon(new ImageIcon("imgs"+sep+"variable.gif"));
    }
    this.add(jl_entries);
  }

}
