package fr.unice.javinspector.GUI.search;

import java.lang.reflect.*;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JPanel;
import fr.unice.javinspector.GUI.util.VerticalFlowLayout;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import java.awt.event.ActionEvent;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import javax.swing.JOptionPane;
import javax.swing.JTextPane;
import fr.unice.javinspector.core.engineSearch.*;

/**
 * <p>Title: NameRegexpCriterion</p>
 * <p>Description: this class filter the member with his name with some regexp</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: UNICE</p>
 * @author Zerbib Gabriel - Charbit Lucas - Galbois Xavier - Gahide Jrme
 * @version 1.0
 */

public class NameRegexpCriterion extends Criterion {

  private String REGEX;
  private String INPUT;
  private static Pattern pattern;
  private static Matcher matcher;

  private JButton helpButton;

  private JComboBox searchText = new JComboBox();
  private JPanel jPanel1 = new JPanel();
  private VerticalFlowLayout verticalFlowLayout1 = new VerticalFlowLayout();
  private JLabel nameofSearch = new JLabel();

  private static JTextPane helpText = new JTextPane();

  /**
   * Construct a new instance of this criterion.
   * This overwrite the name of the criterion
   * This also add additionnal component to take graphicaly the name search.
   */
  public NameRegexpCriterion() {
    name = "nameRegexp";
     REGEX = "";

     // add a component

     helpText.setFocusable(false);

     JPanel moreGraphics = new JPanel();
     searchText.setEditable(true);
     searchText.addItem(new String("Input a pattern to search"));


     searchText.addActionListener(new java.awt.event.ActionListener() {
       public void actionPerformed(ActionEvent e) {
         ValidateQuery_actionPerformed(e);
       }
     });

     nameofSearch.setFont(new java.awt.Font("Serif", 1, 11));
     nameofSearch.setText("name of the method, var to search");

     verticalFlowLayout1.setHgap(10);
     jPanel1.setLayout(verticalFlowLayout1);

     jPanel1.add(nameofSearch, null);
     jPanel1.add(searchText, null);

     helpButton = new JButton("help");


     helpButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        help_actionPerformed(e);
      }
    });


     moreGraphics.add(jPanel1);
     moreGraphics.add(helpButton);


     AdditionalComponent.add(moreGraphics);

  }

  /**
   * Set the name that we want the criterion compare with.
   * @param name the name to compare with the
   */
  public void setNameToCompare(String name){
    REGEX = name;
  }

  /**
   * Here we add a Jcombobox to take the string to compare with.
   * @return Empty if there no additionnal graphics component for a criterion
   * @since 2.0
   */
  public ArrayList isAdditionnalGraphicsComponent(){
    return AdditionalComponent;
  }

  /**
   *
   * @param m the member that we want to check the validity.
   * @return if the Criterion is Valid for this Member
   * @throws SearchEngineException
   */
  public boolean isCriterionValid(Member m)
      throws SearchEngineException{
    boolean res = false;


    INPUT = m.getName();
    pattern = Pattern.compile(REGEX);
    matcher = pattern.matcher(INPUT);

    while(matcher.find()) {
//      System.out.println("I found the text \"" + matcher.group() +
//                         "\" starting at index " + matcher.start() +
//                         " and ending at index " + matcher.end() + ".");
      res = true;
    }


    if( isActive == EngineSearchConst.NOT){
      if( !res )
        return true;
      else
        return false;
    }
    else if( isActive == EngineSearchConst.YES){
      if( res )
        return true;
      else
        return false;
    }
    else if( isActive == EngineSearchConst.DONTMIND){
      // if i don't mind of the result, I always return true
      // logic !!
      return true;
    }
    else{
      throw new SearchEngineException("isCriterionValid : NameRegExpr Criterion : ");
    }

  }

  /**
   * action performed when the search button is cliqued and so
   * a new query submitted.
   * @param e the Action event
   * @throws SearchException
   */
  private void ValidateQuery_actionPerformed(ActionEvent e){

    this.setNameToCompare((String)searchText.getSelectedItem());

    // memory of the search for the combo box
    if(searchText.getSelectedIndex() == -1){
      searchText.addItem(searchText.getSelectedItem());
    }
  }

  String helpS = "Use this text area if you want to use the RegExp Crit\n"+
                 "Exemple : enter [Tt]oto for searching all the Member that contain the pattern toto or Toto\n"+
                 "Here are some of the simple rules that you can use :\n"+
                 "[abc]  a, b, or c (simple class) \n"+
                 "[^abc]  Any character except a, b, or c (negation)  \n"+
                 "[a-zA-Z]  a through z, or A through Z, inclusive (range) \n "+
                 "[a-d[m-p]]  a through d, or m through p: [a-dm-p] (union)  \n "+
                 "[a-z&&[def]]  d, e, or f (intersection)  \n"+
                 "[a-z&&[^bc]]  a through z, except for b and c: [ad-z] (subtraction)  \n"+
                 "[a-z&&[^m-p]]  a through z, and not m through p: [a-lq-z] (subtraction)  \n"+
                 "\\d  A digit: [0-9]  \n"+
                 "\\D  A non-digit: [^0-9]  \n"+
                 "\\s  A whitespace character: [ \\t\\n\\x0B\\f\\r]  \n"+
                 "\\S  A non-whitespace character: [^\\s]  \n"+
                 "\\w  A word character: [a-zA-Z_0-9] \n"+
                 "\\W  A non-word character: [^\\w]  \n"+
                 "^  The beginning of a line \n"+
                 " $  The end of a line \n" +
                 "\\b  A word boundary \n"+
                 "\\B  A non-word boundary \n"+
                 "\\A  The beginning of the input \n"+
                 "\\G  The end of the previous match \n"+
                 "\\Z  The end of the input but for the final terminator, if any \n"+
                 "\\z  The end of the input \n"+
                 "for more information about regexp, go on \n"+
                 "http://java.sun.com/docs/books/tutorial/extra/regex/";




  public void help_actionPerformed(ActionEvent e){
    JOptionPane.showMessageDialog(null, helpS, "Help.", JOptionPane.INFORMATION_MESSAGE);
  }

}
