package fr.unice.javinspector.GUI.search;

import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import fr.unice.javinspector.core.analyser.*;
import fr.unice.javinspector.core.engineSearch.*;
import fr.unice.javinspector.GUI.util.VerticalFlowLayout;
import java.awt.Dimension;
import fr.unice.javinspector.core.engineSearch.advancedEngineSearch.AdvancedEngineSearch;
import fr.unice.javinspector.GUI.search.SearchWindow;

/**
 * <p>Title: SimpleSearch</p>
 * <p>Description: This Gui provide an implementation of a EngineSearch.
 * This can be integreted very simply in a complex GUI because
 * this class extends JPanel.
 * In this search we only use the name criterion so we do not
 * need the group criterion here.
 * This search simply search on a classifier , or an inspector.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: UNICE</p>
 * @author Zerbib Gabriel - Charbit Lucas - Galbois Xavier - Gahide Jrme
 * @version 2.0
 */

public class SimpleSearch extends JPanel{

  private boolean isAdvVisible;

  private JLabel iconeLabel = new JLabel();

  private SearchWindow refParentFrame;

  private static EngineSearch es = null;
  private static GroupCriterion groupCriterion = new NameGroupCriterion();
  private static NameCriterion nc;

  private ArrayList resultOfASearch;

  private AdvancedSearch advSearch;
  private static AdvancedEngineSearch advEngineSearch = null;
  private ImageIcon arrowdown = new ImageIcon(SimpleSearch.class.getResource("/imgs/arrowsdown.gif"));
  private ImageIcon arrowup = new ImageIcon(SimpleSearch.class.getResource("/imgs/arrowsup.gif"));
  private JPanel resultPanelwhereToAdd;
  private VerticalFlowLayout verticalFlowLayoutGlobal = new VerticalFlowLayout();
  private JPanel jPanelSimpleSearch = new JPanel();
  private JPanel jPanelButtonExt = new JPanel();
  private JPanel jPanelAdvSearch = new JPanel();
  private VerticalFlowLayout verticalFlowLayoutSimpleSearch = new VerticalFlowLayout();
  private FlowLayout flowLayoutButton = new FlowLayout();
  private JButton jButtonExtends = new JButton(arrowup);
  private JPanel jPanelButtonSubmit = new JPanel();
  private FlowLayout flowLayoutBouton = new FlowLayout();
  private JButton jButtonSubmitSimpleSearch = new JButton();
  private JPanel jPanelButtonAdv = new JPanel();
  private VerticalFlowLayout verticalFlowLayoutadvSearch = new VerticalFlowLayout();
  private FlowLayout flowLayoutAdvbutton = new FlowLayout();
  private Border border1;

  private JComboBox choiceCombo = new JComboBox();

  /**
   * Construct a new JPanel where a search is implemented
   * @param inspector the inspector to search in
   * @param resultPanel the resultSearch where to add the
   * panel of th resultSearch.
   * @param  frame the search windows
   * @throws NullPointerException
   * @throws ClassCastException
   */
  public SimpleSearch(Inspector inspector, JPanel resultPanel, SearchWindow frame)
      throws NullPointerException, ClassCastException{

    this(resultPanel, frame);


    // add a choice comboBox
    buildChoiceComboBox(inspector);

    if( inspector == null){
      System.err.println("SimpleSearch : error null Pointer for the prameter inspector");
      System.err.println("SimpleSearch : Do nothing aborting");
      throw new NullPointerException("inspector is null in Simplesearch");
    }

    if( advEngineSearch == null){
      advEngineSearch = new AdvancedEngineSearch(inspector);
      addAllcriterion(advEngineSearch);
    }
    else{
      advEngineSearch.setInspector(inspector);
    }

    // make a new Engine search
    if( es == null){
      es = new EngineSearch(inspector);
      groupCriterion = new NameGroupCriterion();
      //bidouille
      Criterion c = (Criterion)groupCriterion.getCriterion().get(0);
      if( c instanceof NameCriterion)
        nc = (NameCriterion)c;
      else{
        throw new ClassCastException("the group was supposed to contains at the first positon a named Criterion");
      }

      es.addGroup(groupCriterion);

    }
    else
      es.setInspector(inspector);


    this.add(iconeLabel);
    if( ! nc.isAdditionnalGraphicsComponent().isEmpty()){
      Iterator iter = nc.isAdditionnalGraphicsComponent().iterator();
      while (iter.hasNext()) {
        Component item = (Component)iter.next();
        jPanelSimpleSearch.add(item);
      }
    }

    try{
      Iterator iter = groupCriterion.getCriterion().iterator();
      while (iter.hasNext()) {
        Criterion item = (Criterion)iter.next();
        item.setUseCriterion(EngineSearchConst.YES);
      }
    }catch(Exception e){
      e.printStackTrace();
      JOptionPane.showMessageDialog(this,
                                    e.getMessage(),
                                    "Error during the setting of the attribute of the criterions."
                                    ,JOptionPane.ERROR_MESSAGE);
    }

    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }

  }

  protected SimpleSearch(JPanel resultPanel, SearchWindow frame)
      throws NullPointerException, ClassCastException{
    super();


    // test for error in paramters.
    if( resultPanel == null){
      System.err.println("SimpleSearch : error null Pointer for the prameter resultPanel");
      System.err.println("SimpleSearch : Do nothing aborting");
      throw new NullPointerException("resultPanel is null in Simplesearch");
    }



    refParentFrame = frame;
    isAdvVisible = false;
    resultPanel.setLayout(new BorderLayout());

    // init the result
    resultOfASearch = new ArrayList();

  }

  /**
   * Construct a new JPanel where a search is implemented
   * @param ca the classifier to search in
   * @param resultPanel the resultSearch where to add the
   * panel of th resultSearch.
   * @param  frame the search windows
   * @throws NullPointerException
   * @throws ClassCastException
   */
  public SimpleSearch(ClassifierAnalyser ca, JPanel resultPanel, SearchWindow frame)
      throws NullPointerException, ClassCastException{

    this(resultPanel,frame);

    if( ca == null){
      System.err.println("SimpleSearch : error null Pointer for the prameter ca");
      System.err.println("SimpleSearch : Do nothing aborting");
      throw new NullPointerException("ca is null in Simplesearch");
    }

    if( advEngineSearch == null){
      advEngineSearch = new AdvancedEngineSearch(ca);
      addAllcriterion(advEngineSearch);
    }
    else{
      advEngineSearch.setClassifierAnalyser(ca);
    }

    // make a new Engine search
    if( es == null){
      es = new EngineSearch(ca);
      groupCriterion = new NameGroupCriterion();
      //bidouille
      Criterion c = (Criterion)groupCriterion.getCriterion().get(0);
      if( c instanceof NameCriterion)
        nc = (NameCriterion)c;
      else{
        throw new ClassCastException("the group was supposed to contains at the first positon a named Criterion");
      }
      es.addGroup(groupCriterion);
    }
    else
      es.setClassifierAnalyser(ca);


    this.add(iconeLabel);
    if( ! nc.isAdditionnalGraphicsComponent().isEmpty()){
      Iterator iter = nc.isAdditionnalGraphicsComponent().iterator();
      while (iter.hasNext()) {
        Component item = (Component)iter.next();
        jPanelSimpleSearch.add(item);
      }
    }

    try{
      Iterator iter = groupCriterion.getCriterion().iterator();
      while (iter.hasNext()) {
        Criterion item = (Criterion)iter.next();
        item.setUseCriterion(EngineSearchConst.YES);
      }
    }catch(Exception e){
      e.printStackTrace();
      JOptionPane.showMessageDialog(this,e.getMessage(),
                                    "Error during the setting of the attribute of the criterions."
                                    ,JOptionPane.ERROR_MESSAGE);
    }

    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }

  }


  /**
   *
   * @param ca the classifier to search in
   * @param icon the icon to put on the GUI
   * @param resultPanel the result panel to put the result Pnael in.
   * @param  frame the search windows
   * @throws NullPointerException
   * @throws ClassCastException
   */
  public SimpleSearch(ClassifierAnalyser ca, Icon icon, JPanel resultPanel, SearchWindow frame)
      throws NullPointerException, ClassCastException {
    this(ca, resultPanel, frame);
    iconeLabel.setIcon(icon);

  }

  /**
   * Set the classifier to search in.
   * @param ca the new classifier to search in.
   * @see EngineSearch#setClassifierAnalyser(ClassifierAnalyser ca) setClassifierAnalyser
   * @throws NullPointerException
   */
  public void setClassifier(ClassifierAnalyser ca)
      throws NullPointerException{
    if( ca == null){
      System.err.println("setClassifier : error null Pointer for the prameter ca");
      System.err.println("setClassifier : Do nothing aborting");
      throw new NullPointerException("ca is null in Simplesearch");
    }
    else
      es.setClassifierAnalyser(ca);
  }



  public void setInspector(Inspector inspector){
    if( inspector == null){
      System.err.println("setClassifier : error null Pointer for the prameter ca");
      System.err.println("setClassifier : Do nothing aborting");
      throw new NullPointerException("ca is null in Simplesearch");
    }
    else
      es.setInspector(inspector);
  }




  /**
   * Set the panel where to add the resultSearch when a search is lunched.
   * @param newPanel the panel where to add the result search
   * @throws NullPointerException
   */
  public void setResulPanel(JPanel newPanel)
      throws NullPointerException{
    if( newPanel == null){
      System.err.println("setResultPanel : error null Pointer for the prameter newPanel");
      System.err.println("setResultPanel : Do nothing aborting");
      throw new NullPointerException("resultPanel is null in Simplesearch");
    }
    else{
      newPanel.setLayout(new BorderLayout());
    }

  }

  /**
   *
   * @return the Engine Search.
   */
  public EngineSearch getEs() {
    return es;
  }

  private void jbInit() throws Exception {
    border1 = BorderFactory.createBevelBorder(BevelBorder.RAISED,Color.white,Color.white,Color.darkGray,new Color(148, 145, 140));
    this.setLayout(verticalFlowLayoutGlobal);
    jPanelSimpleSearch.setLayout(verticalFlowLayoutSimpleSearch);
    jPanelButtonExt.setLayout(flowLayoutButton);
    flowLayoutButton.setAlignment(FlowLayout.RIGHT);
    flowLayoutButton.setHgap(0);
    flowLayoutButton.setVgap(0);
    jPanelSimpleSearch.setBackground(Color.red);
    jPanelButtonExt.setBackground(SystemColor.controlHighlight);
    jPanelAdvSearch.setBackground(Color.orange);
    jPanelAdvSearch.setLayout(verticalFlowLayoutadvSearch);
    this.setBackground(SystemColor.textHighlight);
    jPanelButtonSubmit.setLayout(flowLayoutBouton);
    jButtonSubmitSimpleSearch.setText("search");
    jPanelButtonAdv.setLayout(flowLayoutAdvbutton);
    verticalFlowLayoutGlobal.setHgap(0);
    verticalFlowLayoutGlobal.setVgap(0);
    verticalFlowLayoutSimpleSearch.setHgap(0);
    verticalFlowLayoutSimpleSearch.setVgap(0);
    verticalFlowLayoutadvSearch.setHgap(0);
    verticalFlowLayoutadvSearch.setVgap(0);
    jButtonExtends.setBorder(border1);
    jButtonExtends.setToolTipText("Advanced search");
    jButtonExtends.setRolloverEnabled(true);
    this.add(jPanelSimpleSearch, null);
    jPanelSimpleSearch.add(jPanelButtonSubmit, null);
    this.add(jPanelButtonExt, null);
    this.add(jPanelAdvSearch, null);
    jPanelAdvSearch.add(jPanelButtonAdv, null);
    jPanelButtonExt.add(jButtonExtends, null);
    jPanelButtonSubmit.add(jButtonSubmitSimpleSearch, null);



    jButtonExtends.setBorderPainted(false);
    jButtonExtends.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButtonExtends_actionPerformed(e);
      }
    });

    jButtonExtends.addMouseListener(new MouseListener(){
      public void mouseEntered(MouseEvent e){
        jButtonExtends.setBorderPainted(true);
      }
      public void mouseExited(MouseEvent e){
        jButtonExtends.setBorderPainted(false);
      }
      public void mouseReleased(MouseEvent e){}
      public void mouseClicked(MouseEvent e){}
      public void mousePressed(MouseEvent e){}
    });

    advSearch = new AdvancedSearch(advEngineSearch, refParentFrame.getJtabbedPaneRef(), refParentFrame);
    jPanelButtonAdv.add(advSearch);

    jPanelAdvSearch.setVisible(false);

  }

  private void jButtonExtends_actionPerformed(ActionEvent e) {

    if( isAdvVisible){
      jPanelAdvSearch.setVisible(false);
      jButtonExtends.setIcon(arrowup);
//      jButtonExtends.setText("<<<");
      refParentFrame.setResizable(true);
      refParentFrame.setSize(new Dimension((int)refParentFrame.getSize().getWidth(),
          (int)refParentFrame.getSize().getHeight() - jPanelAdvSearch.getPreferredSize().height));
      refParentFrame.setResizable(false);

      jButtonExtends.setToolTipText("Advenced Search");
      refParentFrame.getButtonSubmitSimple().setEnabled(true);

    }else{
      jPanelAdvSearch.setVisible(true);
      jButtonExtends.setIcon(arrowdown);
//      jButtonExtends.setText(">>>");
      refParentFrame.setResizable(true);
      refParentFrame.setSize(new Dimension((int)refParentFrame.getSize().getWidth(),
          (int)(refParentFrame.getSize().getHeight() + jPanelAdvSearch.getPreferredSize().height)));
      refParentFrame.setResizable(false);
      refParentFrame.getButtonSubmitSimple().setEnabled(false);
      jButtonExtends.setToolTipText("Simple Search");

    }

    isAdvVisible = !isAdvVisible;
  }

  /**
   *
   * @return the submit button for a simple search
   */
  public JButton getJButtonSubmitSimpleSearch() {
    return jButtonSubmitSimpleSearch;
  }

  private void addAllcriterion(AdvancedEngineSearch es){
    es.addCriterion(new NameRegexpCriterion());
    es.addCriterion(new PublicCriterion());
    es.addCriterion(new PrivateCriterion());
    es.addCriterion(new ProtectedCriterion());
    es.addCriterion(new packageCriterion());

    es.addCriterion(new StaticCriterion());
    es.addCriterion(new FinalCriterion());

    es.addCriterion(new InterfaceCriterion());
    es.addCriterion(new AbstractCriterion());
    es.addCriterion(new ClassCriterion());

    es.addCriterion(new ConstructorCriterion());
    es.addCriterion(new MethodCriterion());
    es.addCriterion(new FieldCriterion());


    es.addCriterion(new NativeCriterion());
    es.addCriterion(new StaticCriterion());
    es.addCriterion(new SynchronizedCriterion());
    es.addCriterion(new TransientCriterion());
    es.addCriterion(new VolatileCriterion());
  }


  private void buildChoiceComboBox(Inspector inspector){

    JPanel comboChoicePanel = new JPanel();
    VerticalFlowLayout flowChoiceBox = new VerticalFlowLayout();
    comboChoicePanel.setLayout(flowChoiceBox);
    JLabel titreChoicePanel = new JLabel("Choice your classfier");



    // the code for check all the classifier

    Hashtable tmpHash = inspector.getClassifierAnalysers();
    Enumeration enum = tmpHash.elements();

    // the all item
      choiceCombo.addItem("All Classifier");

    while(enum.hasMoreElements()){
      String item = ((ClassifierAnalyser)enum.nextElement()).getClassAnalysed().getName();
      choiceCombo.addItem(item);

    }


    comboChoicePanel.add(titreChoicePanel);
    comboChoicePanel.add(choiceCombo);
    this.add(comboChoicePanel);

  }

  /**
   *
   * @return the comboBox where are stored the Classifier.
   */
  public JComboBox getChoiceCombo() {
    return choiceCombo;
  }

}
