package fr.unice.javinspector.GUI.util;

import javax.swing.tree.TreeCellRenderer;
import java.awt.Component;
import javax.swing.JTree;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.tree.DefaultMutableTreeNode;
import java.awt.Color;
import fr.unice.javinspector.core.*;
import java.awt.Font;
import java.util.Hashtable;
import javax.swing.Icon;


/**
 *
 * <p>Title: Class TreeCellRendererHeritageView</p>
 * <p>Description: This class change the Look and feel of a JTree</p>
 * <p>Company: UNSA</p>
 * @author Charbit Lucas - Gahide Jrome - Galbois Xavier - Zerbib Gabriel
 * @version 1.0
 */

public class TreeCellRendererHeritageView implements TreeCellRenderer {
  // the hashtable containing the interesting value of the tree
  private Hashtable hashtable;
  private JLabel renderer;
  // the icons
  private ImageIcon nointerface = new ImageIcon(TreeCellRendererHeritageView.class.getResource("/imgs/nointerface.gif"));
  private Icon interfacenode = IconBank.getInterfaceIcon();// ImageIcon(TreeCellRendererHeritageView.class.getResource("/imgs/interfacenode.gif"));
  private ImageIcon racine = new ImageIcon(TreeCellRendererHeritageView.class.getResource("/imgs/racine.gif"));
  private ImageIcon showinterfaces = new ImageIcon(TreeCellRendererHeritageView.class.getResource("/imgs/showinterfaces.gif"));
  private Icon classnode = IconBank.getClassIcon();// ImageIcon(TreeCellRendererHeritageView.class.getResource("/imgs/classnode.gif"));
  private ImageIcon arrowsup = new ImageIcon(TreeCellRendererHeritageView.class.getResource("/imgs/arrowsright.gif"));
  private ImageIcon arrowsdown = new ImageIcon(TreeCellRendererHeritageView.class.getResource("/imgs/arrowsdown.gif"));

  /**
   * The constructor
   * @param hashtable : the values of the tree (the nodes values)
   */
  public TreeCellRendererHeritageView(Hashtable hashtable) {
    this.hashtable = hashtable;
    renderer = new JLabel();
    renderer.setOpaque(true);
  }

  /**
   * This function set the renderer for the current node
   * @param tree : the tree
   * @param value : the value of the node (or leaf)
   * @param selected : return true if the node is selected
   * @param expanded : return true of the node is expanded
   * @param leaf : return true if the node is a leaf
   * @param row : the row of the node
   * @param hasFocus : return true if the node has the focus
   * @return the Component Object of the renderer
   */
  public Component getTreeCellRendererComponent(JTree tree, Object value,
                                                boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
    Color background = selected ? Color.YELLOW : Color.WHITE;
    renderer.setBackground(background);
    String valueName = ((String) ( (DefaultMutableTreeNode) value).getUserObject());
    Class valueClass = (Class) hashtable.get(valueName);
    if(leaf) {
      if(hashtable.get(valueName) == null) {
        renderer.setIcon(nointerface);
      }
      else {
        renderer.setIcon(interfacenode);
      }
    }
    else {
      if (valueName.compareTo("hide/show") == 0) {
        renderer.setIcon(racine);
      }
      else {
        if (hashtable.get(valueName) == null) {
          if (expanded) {
            renderer.setIcon(arrowsdown);
          }
          else {
            renderer.setIcon(arrowsup);
          }
        }
        else {
          if(valueClass.isInterface()) {
            renderer.setIcon(interfacenode);
          }
          else {
            renderer.setIcon(classnode);
          }
        }
      }
    }
    renderer.setText((String)((DefaultMutableTreeNode)value).getUserObject());
    return renderer;
  }
}
