package fr.unice.javinspector.MVC.controler;

import fr.unice.javinspector.MVC.*;
import fr.unice.javinspector.MVC.view.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import fr.unice.javinspector.MVC.manager.Manager;
import javax.swing.tree.TreePath;
import javax.swing.tree.DefaultMutableTreeNode;
import fr.unice.javinspector.core.analyser.ClassifierAnalyser;
import java.util.Hashtable;

/**
 * <p>Title: Class TreeHeritageClass</p>
 * <p>Description: A controler for a TreeHeritageView</p>
 * <p>Company: UNSA </p>
 * @author Charbit Lucas - Gahide Jrome - Galbois Xavier - Zerbib Gabriel
 * @version 1.0
 */
public class TreeHeritageControler extends HeritageControler
    implements ActionListener, MouseListener {
  // the popup menu
  private JPopupMenu jPopupMenu;
  // the two items of the menu, those items can be disable
  private JMenuItem disableItem1;
  private JMenuItem disableItem2;

  /**
   * The default constructor
   */
  public TreeHeritageControler() {}

  /**
   * The constructor
   * @param hm : the father manager
   * @param htv : the view to control
   */
  public TreeHeritageControler(Manager hm, View htv) {
    heritageView = (HeritageView) htv;
    manager = hm;
    createJPopupMenu();
    listenToView();
  }

  /**
   * Needed by the controler interface
   * @return : a string containing the name of the class
   */
  public String getName() {
    return this.getClass().getName();
  }

  /**
   * Needed by the controler interface. This function get the dedired elements in the view
   * and makes the controler listen to them
   */
  public void listenToView() {
    ( (TreeHeritageView) heritageView).getHeritageTree().addMouseListener(this);
  }

  /**
   * Needed by the controler interface. This function tells if the controler is able to
   * control a tupe of a view
   * @param v : the target view
   * @return : true is this controler can control the view, false otherwise
   */
  public boolean isControlable(View v) {
    String shortViewName = v.getName().substring(v.getName().lastIndexOf("."),
                                                 v.getName().length());
    if (shortViewName.compareTo("TreeHeritageView") == 0) {
      return true;
    }
    else {
      return false;
    }
  }

  /**
   * Return the default view that can be controled by the controler
   * @return : a string containing the name of the class of the default view that can be
   * controled by this controler
   */
  public String getDefaultView() {
    return "fr.unice.javinspector.MVC.view.TreeHeritageControler";
  }

  /**
   * Create the JPopupMenu
   */
  public void createJPopupMenu() {
    jPopupMenu = new JPopupMenu("Options");

    disableItem1 = new JMenuItem("Show DAC view");
    disableItem1.setActionCommand("SHOW_DAC_VIEW");
    disableItem1.addActionListener(this);

    disableItem2 = new JMenuItem("Show heritage view");
    disableItem2.setActionCommand("SHOW_HERITAGE_VIEW");
    disableItem2.addActionListener(this);

    jPopupMenu.add(disableItem1);
    jPopupMenu.add(disableItem2);
  }

  /**
   * We catch the MouseEvent. If the user double click, a new view is open,
   * otherwise, we bring the JPopupMenu
   * @param evt : the mouse event
   */
  public void mouseClicked(MouseEvent evt) {
    JTree tree = ( (TreeHeritageView) heritageView).getHeritageTree();
    int selRow = tree.getRowForLocation(evt.getX(), evt.getY());
    TreePath selPath = tree.getPathForLocation(evt.getX(), evt.getY());
    if (selRow != -1) {
      DefaultMutableTreeNode node = (DefaultMutableTreeNode) selPath.
          getLastPathComponent();
      tree.setSelectionPath(selPath);
      if (node != null) {
        Object nodeObject = node.getUserObject();
        String key = (String) nodeObject;
        Hashtable hashtable = ( (TreeHeritageView) heritageView).getHastable();
        if (hashtable.get(key) != null) {
          TreeHeritageView hv = (TreeHeritageView) heritageView;
          if (hv.getHastable().get(key).getClass().getName().compareTo(
              "java.lang.Class") == 0) {
            if ( (evt.getButton() == MouseEvent.BUTTON1) &&
                (evt.getClickCount() == 2)) {
              Class keyClass = (Class) hv.getHastable().get(key);
              if (keyClass != null) {
                ClassifierAnalyser ca = manager.getInspector().
                    getClassifierAnalyser(keyClass.getName());
                manager.addHeritageView(ca);
              }
            }
            else if (evt.getButton() == MouseEvent.BUTTON3) {
              disableItem1.setEnabled(true);
	      disableItem2.setEnabled(true);
              jPopupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
              tree.setSelectionPath(selPath);
            }
          }
        }
      }
    }
    else {
      if (evt.getButton() == MouseEvent.BUTTON3) {
        jPopupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
        disableItem1.setEnabled(false);
	disableItem2.setEnabled(false);
      }
    }
  }

  /**
   * Needed by the MouseListened interface
   * @param evt : the mouse event
   */
  public void mousePressed(MouseEvent evt) {};
  public void mouseEntered(MouseEvent evt) {};
  public void mouseExited(MouseEvent evt) {};
  public void mouseReleased(MouseEvent evt) {};

  /**
   * We cacth the action events coming from the JPopupMenu.
   * @param evt : the action event
   */
  public void actionPerformed(ActionEvent evt) {
    // open the heitage view of the selected classifier
    if (evt.getActionCommand().compareTo("SHOW_DAC_VIEW") == 0) {
      JTree tree = ( (TreeHeritageView) heritageView).getHeritageTree();
      DefaultMutableTreeNode node =
          (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
      Object nodeObject = node.getUserObject();
      String key = (String) nodeObject;
      Class keyClass =
          (Class) ( (TreeHeritageView) heritageView).getHastable().get(key);
      ClassifierAnalyser ca = getManager().getInspector().getClassifierAnalyser(
          keyClass.getName());
      getManager().addDetailView(ca);
    }
    // open the DAC view of the selected classifier
    else if(evt.getActionCommand().compareTo("SHOW_HERITAGE_VIEW") == 0) {
      JTree tree = ( (TreeHeritageView) heritageView).getHeritageTree();
      DefaultMutableTreeNode node =
          (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
      Object nodeObject = node.getUserObject();
      String key = (String) nodeObject;
      Class keyClass =
          (Class) ( (TreeHeritageView) heritageView).getHastable().get(key);
      ClassifierAnalyser ca = getManager().getInspector().getClassifierAnalyser(
          keyClass.getName());
      getManager().addHeritageView(ca);
    }
  }
}
