package fr.unice.javinspector.MVC.manager;

import fr.unice.javinspector.core.analyser.Inspector;
import fr.unice.javinspector.core.analyser.ClassifierAnalyser;
import java.awt.Container;
import fr.unice.javinspector.MVC.view.View;
import fr.unice.javinspector.plugin.PluginTypes;

/**
 * <p>Title: JavInspector</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Jerme GAHIDE - Lucas CHARBIT - Gabriel ZERBIB - Xavier GALBOIS
 * @version 1.0
 */

public abstract class AbstractManager implements Manager {
  private Manager topManager;

// View and controler implementation not really usefull
  public void listenToView(){}
  public View getView() {
    return this;
  }
  public ClassifierAnalyser getClassifierAnalyser(){return null;}
  public void setClassifierAnalyser(ClassifierAnalyser ca){}

  public String getDefaultView(){return this.getName();}

  public int getType(){
    return PluginTypes.MANAGER;
  }

  /**
   * default constructor builds an abstract manager
   * @param parent the parent manager
   */
  public AbstractManager(Manager parent){
    topManager = parent;
  }

  public boolean isControlable(View v){
    return (v.getName().compareTo(this.getName()) == 0);
  }

  /**
   * default implementation : forwards the event to its parent manager
   * @return the inspector
   */
  public Inspector getInspector() {
    return topManager.getInspector();
  }

  /**
   * default implementation : forwards the event to its parent manager
   * @param type type of the plugin
   * @return the Class object of the plugin
   */
  public Class getPlugin(int type) {
    return topManager.getPlugin(type);
  }
  /**
   * default implementation : forwards the event to its parent manager
   * @param ca the data model to represent
   */
  public void addHeritageView(ClassifierAnalyser ca) {
    topManager.addHeritageView(ca);
  }

  /**
   * default implementation : forwards the event to its parent manager
   * @param ca the data model to represent
   */
  public void addDetailView(ClassifierAnalyser ca) {
    topManager.addDetailView(ca);
  }

  /**
   * default implementation : forwards the event to its parent manager
   * @param ca the data model to represent
   */
  public void addCompositionView(ClassifierAnalyser ca) {
    topManager.addCompositionView(ca);
  }

  /**
   * default implementation : forwards the event to its parent manager
   * @param ca the data model to represent
   */
  public void addSearchView(ClassifierAnalyser ca) {
    topManager.addSearchView(ca);
  }

  /**
   * default implementation : forwards the event to its parent manager
   * @return the topmanager
   */
  public AbstractTopManager getTopManager() {
    return topManager.getTopManager();
  }
}
