package fr.unice.javinspector.MVC.manager;

import java.awt.*;
import javax.swing.*;
import fr.unice.javinspector.core.analyser.*;
import fr.unice.javinspector.MVC.view.*;
import fr.unice.javinspector.MVC.controler.TreeHeritageControler;
import fr.unice.javinspector.MVC.controler.GraphDetailControler;
import java.util.Hashtable;
//import fr.unice.javinspector.MVC.PluginSettings;

/**
 * <p>Title: Class JSplitPaneManager</p>
 * <p>Description: This class implements a TopManager with two sub-managers. The two sub-managers
 * are in split pane.</p>
 * <p>Company: UNSA</p>
 * @author Charbit Lucas - Gahide Jrome - Galbois Xavier - Zerbib Gabriel
 * @version 1.0
 */
public class SplitPaneTopManager extends AbstractTopManager {
  /**
   * The first sub-manager is an heritageManager, for the heritage views
   */
  private Manager heritageManager;
  /**
   * The second manager is a dacManager, for teh composition and details views
   */
  private Manager dacManager;
  /**
   * the main split pane
   */
  JSplitPane jSplitPane1 = new JSplitPane();
  /**
   * In order to readjust the spliPane when the window is resized
   */
  private JPanel globalPanel = new JPanel(){
    public void paint(Graphics gc){
      if (jSplitPane1 != null) jSplitPane1.setDividerLocation(0.3);
      super.paint(gc);
    }
  };
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel jPanelCenter = new JPanel();
  JPanel jPanelNorth = new JPanel();
  FlowLayout flowLayout2 = new FlowLayout();
  BorderLayout borderLayout2 = new BorderLayout();

  /**
   * the constructor
   * @param inspector : the inspector of this manager
   * @param pluginSettings : the plugins settings for the class the Manager will have to instanciate
   */
  public SplitPaneTopManager(Inspector inspector, Hashtable pluginSettings) {
    super(inspector, pluginSettings);
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    heritageManager = new TabbedPaneHeritageManager(this);
    dacManager = new TabbedPaneDACManager(this);
    jSplitPane1.setLeftComponent(heritageManager.getViewContainer());
    jSplitPane1.setRightComponent(dacManager.getViewContainer());
  }

  public int getType() {
    /**
     * @todo : penser aux constantes pour les plugins
     */
    throw new java.lang.UnsupportedOperationException("Method getName() not yet implemented.");
  }

  /**
   * This function return the name of this class
   * @return : the name of this class
   */
  public String getName() {
    return this.getClass().getName();
  }

  public boolean isControlable(View v) {
    /**@todo Implement this fr.unice.javinspector.MVC.controler.Controler abstract method*/
    throw new java.lang.UnsupportedOperationException("Method isControlable() not yet implemented.");
  }

  /**
   * Retuns the main container of this manager
   * @return : a container
   */
  public Container getViewContainer() {
    return globalPanel;
  }

  // **** DON'T EDIT *****
  private void jbInit() throws Exception {
    globalPanel.setLayout(borderLayout1);
    jPanelCenter.setLayout(borderLayout2);
    flowLayout2.setAlignment(FlowLayout.LEFT);
    flowLayout2.setHgap(5);
    flowLayout2.setVgap(2);
    jSplitPane1.setBorder(null);
    jSplitPane1.setDoubleBuffered(true);
    jSplitPane1.setDividerSize(5);
    jSplitPane1.setResizeWeight(0.0);
    globalPanel.setEnabled(true);
    globalPanel.setBorder(null);
    globalPanel.add(jPanelCenter, BorderLayout.CENTER);
    jPanelCenter.add(jSplitPane1, BorderLayout.CENTER);
  }

  /**
   * This function use the delegation and allow to manager to add a new heritage view
   * @param ca : the classiferAnalyser for the new view
   */
  public void addHeritageView(ClassifierAnalyser ca){
    heritageManager.addHeritageView(ca);
  }

  /**
   * This function use the delegation and allow to manager to add a new detail view
   * @param ca : the classiferAnalyser for the new view
   */
  public void addDetailView(ClassifierAnalyser ca){
    dacManager.addDetailView(ca);
  }

  /**
   * This function use the delegation and allow to manager to add a new composition view
   * @param ca : the classiferAnalyser for the new view
   */
  public void addCompositionView(ClassifierAnalyser ca){
    dacManager.addCompositionView(ca);
  }

  /**
   * This function use the delegation and allow to manager to add a new search view
   * This has no been implemented yet, the search results are not views
   * @param ca : the classiferAnalyser for the new view
   */
  public void addSearchView(ClassifierAnalyser ca){
    /**@todo Implement this fr.unice.javinspector.MVC.Manager.SplitPaneTopManager method*/
    throw new java.lang.UnsupportedOperationException("Method addSearchView() not yet implemented.");
  }

  /**
   * This function returns the split pane
   * @return the main splitpane
   */
  public JSplitPane getJSplitPane1() {
    return jSplitPane1;
  }
}
