package fr.unice.javinspector.MVC.view;

import java.awt.Container;
import fr.unice.javinspector.core.analyser.ClassifierAnalyser;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

/**
 * <p>Title: Class AbstractView</p>
 * <p>Description: This class implements the View interface. This class needed to
 * extended in order to build an heritage view.</p>
 * <p>Company: UNSA</p>
 * @author Jerme GAHIDE - Lucas CHARBIT - Gabriel ZERBIB - Xavier GALBOIS
 * @version 1.0
 */

public abstract class AbstractView implements View {
  /**
   * The classifier analyser represented by this view
   */
  protected ClassifierAnalyser classifierAnalyser;

  /**
   * The default controler
   */
  public AbstractView(){
  }

  /**
   * retun the classifierAnalyser related to this view
   * @return : a classifierAnalsyer
   */
  public ClassifierAnalyser getClassifierAnalyser(){
    return classifierAnalyser;
  }

  /**
   * Set the current classifierAnalsyer of this view
   * @param ca : the new classifierAnalsyer
   */
  public void setClassifierAnalyser(ClassifierAnalyser ca){
    classifierAnalyser = ca;
  }

  /**
   * Create a new view, with the specficied view class ans the specified classifierAnalyser
   * @param theView : the plugin view class selected
   * @param ca : the classifierAnalyser
   * @return : a new View
   */
  public static View create(Class theView, ClassifierAnalyser ca){
    View res = null;
        Class[] caArgsClass = new Class[] {ClassifierAnalyser.class};
            Object[] caArgs = new Object[] {ca};
            Constructor caArgsConstructor;
           try {
              caArgsConstructor = theView.getConstructor(caArgsClass);
              res = (View) createObject(caArgsConstructor, caArgs);
          } catch (NoSuchMethodException e) {
              System.err.println(e);
          }
          return res;
  }

  /**
   * This function is use by the create function to instanciate the object
   * @param constructor : the constructor
   * @param arguments : the constructor arguments
   * @return : a new Object
   */
  private static Object createObject(Constructor constructor,
                                     Object[] arguments) {

    Object object = null;
    try {
      object = constructor.newInstance(arguments);
      return object;
    } catch (InstantiationException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    } catch (IllegalArgumentException e) {
      e.printStackTrace();
    } catch (InvocationTargetException e) {
      e.getTargetException().printStackTrace();
    }
    return object;
  }
}
