package fr.unice.javinspector.MVC.view;

import fr.unice.javinspector.MVC.*;
import java.awt.*;
import javax.swing.*;
import java.util.Hashtable;
import fr.unice.javinspector.core.analyser.ClassifierAnalyser;
import javax.swing.tree.DefaultMutableTreeNode;
import java.util.ArrayList;
import fr.unice.javinspector.GUI.util.TreeCellRendererHeritageView;
import javax.swing.border.*;
import fr.unice.javinspector.GUI.util.JTabbedPaneWithCloseIcons;

/**
 * <p>Title: Class TreeHeritageView</p>
 * <p>Description: This class implements an heritage view for a classifierAnalyser with a JTree. This
 * class is an heritage vew so it extends the class HeritageView.</p>
 * <p>Company: UNSA</p>
 * @author Charbit Lucas - Gahide Jrome - Galbois Xavier - Zerbib Gabriel
 * @version 1.0
 */
public class TreeHeritageView extends HeritageView {
  /**
   *  The classifierAnalyser of the view
   */
  protected ClassifierAnalyser classifierAnalyser;
  // **** GUI OBJECTS ****
  protected JPanel globalPanel = new JPanel();
  protected JTree heritageTree;
  protected TreeCellRendererHeritageView cellRenderer;
  protected Hashtable hashtable;
  protected BorderLayout borderLayout1 = new BorderLayout();
  protected JPanel jPanelTree = new JPanel();
  protected JPanel jPanelStatut = new JPanel();
  protected FlowLayout flowLayout1 = new FlowLayout();
  protected JPanel centerPanel = new JPanel();
  protected JScrollPane jScrollPane;
  protected BorderLayout borderLayout3 = new BorderLayout();
  protected BorderLayout borderLayout4 = new BorderLayout();
  protected Border border2;
  protected FlowLayout flowLayout2 = new FlowLayout();
  protected JLabel jLabelClassifierNameValue = new JLabel();

  /**
   * The default constructor
   */
  public TreeHeritageView(){}

  /**
   * The constructor
   * @param classifierAnalyser : the classifierAnalyser represented by the view
   */
  public TreeHeritageView(ClassifierAnalyser classifierAnalyser) {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    this.classifierAnalyser = classifierAnalyser;
    hashtable = new Hashtable();
    jLabelClassifierNameValue.setText(classifierAnalyser.getClassAnalysed().getName());
    updateView();
    jScrollPane = new JScrollPane(centerPanel);
    jScrollPane.setBorder(null);
    globalPanel.add(jScrollPane, BorderLayout.CENTER);
    globalPanel.add(jPanelStatut, BorderLayout.SOUTH);
  }

  /**
   * Needed by the Plugin interface
   * @return : the name of this class
   */
  public String getName() {
    return this.getClass().getName();
  }

  /**
   * Needed by the View interface
   * @return : a container
   */
  public Container getViewContainer() {
    return globalPanel;
  }

  public JTree getHeritageTree() {
    return heritageTree;
  }

  public Hashtable getHastable() {
    return hashtable;
  }

  /**
   * Construct the view
   */
  public void updateView() {
    constructTree();
    cellRenderer = new TreeCellRendererHeritageView(hashtable);
    heritageTree.setCellRenderer(cellRenderer);
    heritageTree.putClientProperty("JTree.lineStyle", "Angled");
    centerPanel.add(heritageTree, BorderLayout.CENTER);
  }

  /**
   * Construct the tree for the view
   */
  public void constructTree() {
    String racineText = "";
    DefaultMutableTreeNode racine = null;
    if(!classifierAnalyser.getClassAnalysed().isInterface()) {
      racineText = "hide/show";//"class " + classifierAnalyser.getClassAnalysed().getName();
      racine = new DefaultMutableTreeNode(racineText);
      constructClassTree(racine);
    }
    else if(classifierAnalyser.getClassAnalysed().isInterface()){
      racineText = "interface " + classifierAnalyser.getClassAnalysed().getName();
      racine = new DefaultMutableTreeNode(racineText);
      DefaultMutableTreeNode interfaceNode = constructInterfaceTree();
      racine.add(interfaceNode);
    }
    heritageTree = new JTree(racine);
  }

  /**
   * Get all the inertface node if the classifier is not an interface
   * @param currentCA : the classifierAnalyser
   * @return : a node for the tree
   */
  public DefaultMutableTreeNode getClassInterfaceNode(ClassifierAnalyser currentCA) {
    ClassifierAnalyser[] classInterface = currentCA.getInterfacesClassifier();
    DefaultMutableTreeNode firstNode = null;
    DefaultMutableTreeNode secondNode = null;
    String key = "";
    Class keyClass = null;
    if((classInterface == null) || (classInterface.length < 1)) {
      firstNode = new DefaultMutableTreeNode("No interfaces implemented");
    }
    else {
      firstNode = new DefaultMutableTreeNode("Interfaces implemented");
      for(int i=0; i<classInterface.length; i++) {
        key = classInterface[i].getClassAnalysed().getName();
        keyClass = classInterface[i].getClassAnalysed();
        secondNode = new DefaultMutableTreeNode(key);
        firstNode.add(secondNode);
        hashtable.put(key, keyClass);
      }
    }
    return firstNode;
  }

  /**
   * Construct the node for the tree when the classifier is a class
   * @param racine
   */
  public void constructClassTree(DefaultMutableTreeNode racine) {
    ArrayList superClass = classifierAnalyser.getHeritageClassifier();
    DefaultMutableTreeNode firstNode = null;
    String key = "";
    Class keyClass = null;
    if((superClass == null) || (superClass.size() < 1)) {
      firstNode = new DefaultMutableTreeNode("No class extended");
      racine.add(firstNode);
    }
    else {
      for(int i = superClass.size() - 1; i >= 0; i--) {
        ClassifierAnalyser currentCA = (ClassifierAnalyser)superClass.get(i);
        key = currentCA.getClassAnalysed().getName();
        keyClass = currentCA.getClassAnalysed();
        firstNode = new DefaultMutableTreeNode(key);
        firstNode.add(getClassInterfaceNode(currentCA));
        hashtable.put(key, keyClass);
        racine.add(firstNode);
      }
    }
  }

  /**
   * Construct the node for the tree when the classifier is an interface
   * @return the constructes node
   */
  public DefaultMutableTreeNode constructInterfaceTree() {
    ClassifierAnalyser[] superInterface = classifierAnalyser.
        getAllSuperInterfacesClassifier();
    String key = classifierAnalyser.getClassAnalysed().getName();
    Class keyClass = classifierAnalyser.getClassAnalysed();
    DefaultMutableTreeNode firstNode = new DefaultMutableTreeNode(key);
    DefaultMutableTreeNode secondNode = null;
    DefaultMutableTreeNode thirdNode = null;
    hashtable.put(key, keyClass);
    if ( (superInterface == null) || (superInterface.length < 1)) {
      secondNode = new DefaultMutableTreeNode("No interfaces extended");
    }
    else {
      secondNode = new DefaultMutableTreeNode("Interfaces extended");
      for (int i = 0; i < superInterface.length; i++) {
        key = superInterface[i].getClassAnalysed().getName();
        keyClass = superInterface[i].getClassAnalysed();
        hashtable.put(key, keyClass);
        thirdNode = new DefaultMutableTreeNode(key);
        secondNode.add(thirdNode);
      }
    }
    firstNode.add(secondNode);
    return firstNode;
  }

  /**
   * Gui Initialisation
   * @throws Exception
   */
  private void jbInit() throws Exception {
    border2 = BorderFactory.createCompoundBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED,Color.white,Color.white,new Color(124, 124, 124),new Color(178, 178, 178)),BorderFactory.createEmptyBorder(5,5,5,5));
    globalPanel.setLayout(borderLayout1);
    centerPanel.setBackground(Color.white);
    centerPanel.setBorder(border2);
    centerPanel.setLayout(borderLayout4);
    jPanelTree.setBackground(Color.white);
    jPanelTree.setBorder(border2);
    jPanelTree.setLayout(flowLayout1);
    flowLayout1.setAlignment(FlowLayout.RIGHT);
    jPanelStatut.setLayout(flowLayout2);
    jLabelClassifierNameValue.setBorder(null);
    jLabelClassifierNameValue.setText("");
    flowLayout2.setHgap(2);
    flowLayout2.setVgap(0);
    jPanelStatut.setBorder(BorderFactory.createLoweredBevelBorder());
    centerPanel.add(jPanelTree,  BorderLayout.CENTER);
    jPanelStatut.add(jLabelClassifierNameValue, null);
  }
}
