package fr.unice.javinspector.core.analyser;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
/**
 * <p>Title: FieldAnalyser</p>
 * <p>Description: This class allow to analysed a field.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author unascribed
 * @version 1.0
 */
public class FieldAnalyser {
  private Field field;
  private Object value = null;
  private int modifier;
  private TypeClass typeClass;
  private String fullName;
  private ClassifierAnalyser classifierAnalyser;


  /**
   * Construct a new FieldAnalyser
   * @param field the field to analyse
   * @param cl the classifier analyser of this
   */
  public FieldAnalyser(Field field,ClassifierAnalyser cl) {
    field.setAccessible(true);
    this.field = field;
    this.classifierAnalyser = cl;
    modifier = field.getModifiers();
    typeClass = classifierAnalyser.formatParametre(field.getType());

    if(isStatic()){
      computeValue();
    }

  }

  /**
   *
   * @return the full name of the field
   */
  public String getFullName(){
    if(!isStatic()){
      return getStringModifier() + " " + getTypeName()+ " " +  field.getName() + "; ";
    }
    computeValue();
    return getStringModifier() + " " + getTypeName() + " " + field.getName() + " = " + value + "; ";
  }

  /**
   *
   * @return a string representation of the modifier
   */
  public String getStringModifier(){
    return Modifier.toString(modifier);
  }

  /**
   *
   * @return true if the modifier is static
   */
  public boolean isStatic(){
    return Modifier.isStatic(modifier);
  }

  /**
   *
   * @return the name of the field
   */
  public String getName(){
    return field.getName();
  }

  /**
   *
   * @return a string that represent the type of the string.
   */
  public String getTypeName(){
    return typeClass.getName();
  }

  /**
   * compute the value of the field.
   */
  public void computeValue(){
    if(field.getType().isPrimitive()){
      Class fieldClass = field.getType();
      try {
        if(fieldClass.getName().compareTo("int") == 0) {
         // System.out.println("type:int value:" + field.getInt(field));
          value = Integer.valueOf(String.valueOf(field.getInt(field)));
          return;
        }
        else if(fieldClass.getName().compareTo("double") == 0) {
          //System.out.println("type:double value:" + field.getDouble(field));
          value =  Double.valueOf(String.valueOf(field.getDouble(field)));
        }
        else if(fieldClass.getName().compareTo("float") == 0) {
          //System.out.println("type:float value:" + field.getFloat(field));
          value =  Float.valueOf(String.valueOf(field.getFloat(field)));
        }
        else if(fieldClass.getName().compareTo("long") == 0) {
          value =  Long.valueOf(String.valueOf(field.getLong(field)));
        }
        else if(fieldClass.getName().compareTo("char") == 0) {
          //System.out.println("type:char value:" + field.getChar(field));
          value = Character.toString(field.getChar(field));
        }
        else if(fieldClass.getName().compareTo("boolean") == 0) {
          //System.out.println("type:char value:" + field.getBoolean(field));
          value = Boolean.toString(field.getBoolean(field));
        }
        else if(fieldClass.getName().compareTo("short") == 0) {
          //System.out.println("type:char value:" + field.getShort(field));
          value = Short.toString(field.getShort(field));
        }

        else value = "PRIMITIVE TYPE UNKNOWN";
      }
      catch(Exception e) {
        e.printStackTrace();
        value =  "TYPE PRIMITIF INCONNU .... 2";
      }
    }
    else {
      try {
        Object res = field.get(null);
        if (res == null) {
          value = null;
          return;
        }
        else if (res != null && field.getType().getName().compareTo("java.lang.String") != 0){
          value = "unknown value";
          return;
        }
        else if (res!=null && field.getType().getName().compareTo("java.lang.String") == 0){
          //System.out.println("type:" + res.getClass() + " value:" + res);
          value =  "\"" + String.valueOf(res) + "\"";
          return;
        }
      }
      catch(Exception e) {
        e.printStackTrace();
        value =  "EERRREUR BUG SYSTEM";
      }
    }
  }


  /**
   *
   * @return a String representation.
   */
  public String toString(){
    return "toto";
  }

  /**
   *
   * @return the field.
   */
  public Field getField() {
    return field;
  }

  /**
   *
   * @return the modifier for this field.
   */
  public int getModifier() {
    return modifier;
  }

  /**
   *
   * @return the type class of the field.
   */
  public TypeClass getTypeClass() {
    return typeClass;
  }

  /**
   *
   * @return the value of the field.
   */
  public Object getValue() {
    computeValue();
    return value;
  }

}
