package fr.unice.javinspector.core.engineSearch;

import fr.unice.javinspector.core.analyser.Inspector;
import java.util.ArrayList;
import java.util.Iterator;
import fr.unice.javinspector.core.analyser.ClassifierAnalyser;
import java.lang.reflect.Member;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

/**
 * <p>Title: EngineSearch.</p>
 * <p>Description: This class construct an Engine to make some search on
 * an Inspector</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: UNICE</p>
 * @author Zerbib Gabriel - Charbit Lucas - Galbois Xavier - Gahide Jrme
 * @version 1.0
 * @see fr.unice.javinspector.core.analyser.ClassifierAnalyser
 */



public class EngineSearch {

  private Inspector inspector;

  private ClassifierAnalyser classifier;

  /**
   * TYPE_OF_SEARCH = -1 by default
   * TYPE_OF_SEARCH = 0 on a inspector search
   * TYPE_OF_SEARCH = 1 on a ClassifierAnalyser search
   * @see fr.unice.javinspector.core.analyser.ClassifierAnalyser
   * @see fr.unice.javinspector.core.analyser.Inspector
   */
  private int TYPE_OF_SEARCH = -1;

  private ArrayList groupSearch;

  /**
   * Construct a new search based on the inspector
   * given.
   * @param inspector the inspector.
   */
  public EngineSearch(Inspector inspector){
    this.inspector = inspector;
    groupSearch = new ArrayList();
    TYPE_OF_SEARCH = 0;
  }

  /**
   * Construct a new search based on
   * the classifier analyser given.
   * @param ca the classifier analyser to search on
   */
  public EngineSearch(ClassifierAnalyser ca){
      this.classifier = ca;
      groupSearch = new ArrayList();
      TYPE_OF_SEARCH = 1;
  }

  /**
   * To change the inspector if necessary
   * @param inspector the new inspector
   */
  public void setInspector(Inspector inspector) {
    this.inspector = inspector;
    TYPE_OF_SEARCH = 0;
  }

  /**
   * To change the classifier analyser if necessary.
   * @param ca the new classifier analyser
   */
  public void setClassifierAnalyser(ClassifierAnalyser ca) {
    this.classifier = ca;
    TYPE_OF_SEARCH = 1;
  }

  /**
   * Add a group of criterion to the search
   * @param gcriterion the group to add
   * @see fr.unice.javinspector.core.engineSearch.GroupCriterion
   * @see fr.unice.javinspector.core.engineSearch.Criterion
   */
  public void addGroup(GroupCriterion gcriterion){
    groupSearch.add(gcriterion);
  }

  /**
   * Carry out the search with the Group criterions.
   * @return an ArrayList of Member that contains the result.
   * @throws SearchEngineException
   */
  public ArrayList search()
      throws SearchEngineException{
    ArrayList res = new ArrayList();

    if( TYPE_OF_SEARCH == 0)
      res = searchInspector();
    else if( TYPE_OF_SEARCH == 1)
      res = searchClassifieurAnalyser();
    else{
      throw new SearchEngineException(
                "Invalid type of search : aborting");
    }



    return res;
  }

  /**
   * specialized function that search in a classifier.
   * @return the ArrayList of the result. It's an ArrayList of Member.
   * @throws SearchEngineException if an error occur.
   */
  private ArrayList searchClassifieurAnalyser()
      throws SearchEngineException{
    ArrayList res = new ArrayList();
    ClassifierAnalyser advAnalysor = this.classifier;


    // check the constructor
    Constructor[] constr = advAnalysor.getClassAnalysed().getDeclaredConstructors();

    for( int i = 0 ; i < constr.length ; i++){
      boolean localRes = true;
      Iterator iter_groupSearch = groupSearch.iterator();
      while (iter_groupSearch.hasNext()) {
        GroupCriterion group = (GroupCriterion)iter_groupSearch.next();

        try{
          localRes = localRes && group.evaluateCriterion(constr[i]);
        }catch(SearchEngineException see){
          System.err.println(see);
          see.printStackTrace();
          throw new SearchEngineException(
              "An internal error in the search occur : aborting");
        }
      }
      if(localRes)  res.add(constr[i]);
    }

    // check the fields
    Field[] field = advAnalysor.getClassAnalysed().getDeclaredFields();

    for( int i = 0 ; i < field.length ; i++){
      boolean localRes = true;
      Iterator iter_groupSearch = groupSearch.iterator();
      while (iter_groupSearch.hasNext()) {
        GroupCriterion group = (GroupCriterion)iter_groupSearch.next();

        try{
          localRes = localRes && group.evaluateCriterion(field[i]);
        }catch(SearchEngineException see){
          System.err.println(see);
          see.printStackTrace();
          throw new SearchEngineException(
              "An internal error in the search occur : abborting");
        }
      }
      if(localRes)  res.add(field[i]);
    }

    // check the Methods
    Method[] method = advAnalysor.getClassAnalysed().getDeclaredMethods();

    for( int i = 0 ; i < method.length ; i++){
      boolean localRes = true;
      Iterator iter_groupSearch = groupSearch.iterator();
      while (iter_groupSearch.hasNext()) {
        GroupCriterion group = (GroupCriterion)iter_groupSearch.next();

        try{
          localRes = localRes && group.evaluateCriterion(method[i]);
        }catch(SearchEngineException see){
          System.err.println(see);
          see.printStackTrace();
          throw new SearchEngineException(
              "An internal error in the search occur : aborting");
        }
      }
      if(localRes){
        /** voir boom exeception */
        res.add(method[i]);
      }
    }
    return res;
  }

  /**
   * specialized function that search in a classifier.
   * @return the ArrayList of the result. It's an ArrayList of Member.
   * @throws SearchEngineException if an error occur.
   */
  private ArrayList searchInspector()
      throws SearchEngineException{
    ArrayList res = new ArrayList();


    Iterator iter_inspector = inspector.getClassifierAnalyserIterator();
    while (iter_inspector.hasNext()) {
      ClassifierAnalyser advAnalysor = (ClassifierAnalyser)iter_inspector.next();

      // check the constructor
      Constructor[] constr = advAnalysor.getClassAnalysed().getDeclaredConstructors();

      for( int i = 0 ; i < constr.length ; i++){
        boolean localRes = true;
        Iterator iter_groupSearch = groupSearch.iterator();
        while (iter_groupSearch.hasNext()) {
          GroupCriterion group = (GroupCriterion)iter_groupSearch.next();

          try{
            localRes = localRes && group.evaluateCriterion(constr[i]);
          }catch(SearchEngineException see){
            System.err.println(see);
            see.printStackTrace();
            throw new SearchEngineException(
                "An internal error in the search occur : aborting");
          }
        }
        if(localRes)  res.add(constr[i]);
      }

      // check the fields
      Field[] field = advAnalysor.getClassAnalysed().getDeclaredFields();

      for( int i = 0 ; i < field.length ; i++){
        boolean localRes = true;
        Iterator iter_groupSearch = groupSearch.iterator();
        while (iter_groupSearch.hasNext()) {
          GroupCriterion group = (GroupCriterion)iter_groupSearch.next();

          try{
            localRes = localRes && group.evaluateCriterion(field[i]);
          }catch(SearchEngineException see){
            System.err.println(see);
            see.printStackTrace();
            throw new SearchEngineException(
                "An internal error in the search occur : abborting");
          }
        }
        if(localRes)  res.add(field[i]);
      }

      // check the Methods
      Method[] method = advAnalysor.getClassAnalysed().getDeclaredMethods();

      for( int i = 0 ; i < method.length ; i++){
        boolean localRes = true;
        Iterator iter_groupSearch = groupSearch.iterator();
        while (iter_groupSearch.hasNext()) {
          GroupCriterion group = (GroupCriterion)iter_groupSearch.next();

          try{
            localRes = localRes && group.evaluateCriterion(method[i]);
          }catch(SearchEngineException see){
            System.err.println(see);
            see.printStackTrace();
            throw new SearchEngineException(
                "An internal error in the search occur : aborting");
          }
        }
        if(localRes){
          /** voir boom exeception */
          res.add(method[i]);
        }
      }
    }
    return res;
  }

  /**
   *
   * @return the ArrayList of the group of criterion
   * @see fr.unice.javinspector.core.engineSearch.GroupCriterion
   * @see fr.unice.javinspector.core.engineSearch.Criterion
   */
  public ArrayList getGroupSearch() {
    return groupSearch;
  }

}
