package fr.unice.javinspector.core.engineSearch;

import java.util.ArrayList;
import java.lang.reflect.Member;
import java.util.Iterator;


/**
 * <p>Title: GroupCriterion</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: UNICE </p>
 * @author Zerbib Gabriel, Charbit Lucas, Galbois Xavier Gahide Jrme
 * @version 1.0
 */

public abstract class GroupCriterion{

  /**
   * The name of the group criterion
   */
  protected String name = "Undefined";

  /**
   * The list of the criterions that are contains by this group.
   */
  protected ArrayList criterionList = new ArrayList();

  /**
   * @return the name of the group of criterion
   */
  public String getName(){
    return name;
  }

  /**
   * Add a criterion to the group
   * @param c the Criterion to add.
   */
  public void addCriterion(Criterion c){
    criterionList.add(c);
  }

  /**
   * This the mediator for selecting a criterion. Thank to this method,
   * we are able to apply simple rukes like : "only one criterion can be
   * selected at once at the state YES"
   * @deprecated the mediator has not been implemented.
   * @param c the criterion to set.
   * @param state the state to set c
   * @see EngineSearchConst
   */
  public abstract void selectStatesCriterion(Criterion c, int state);

  /**
   *
   * @deprecated the mediator has not been implemented.
   * @param c the criterion to set
   * @param ll  the logical Value ( AND ORN NOT )
   * @see EngineSearchConst
   */
  public abstract void selectLogicalLinkCriterion(Criterion c, int ll);

  /**
   * @return all the criterion of the group
   */
  public ArrayList getCriterion(){
    return criterionList;
  }


  /**
   * Evaluate if a member is a result for the search in this group of criterion.
   * @param m the member to evaluate
   * @return true if the evaluation of all the criterion beetween them are true.
   * @throws SearchEngineException
   * @see Criterion#isCriterionValid(Member m)
   */
  public boolean evaluateCriterion(Member m)
  throws SearchEngineException{
    boolean res = true;

    Iterator iter = criterionList.iterator();
    while (iter.hasNext()) {
      Criterion item = (Criterion)iter.next();
      if( item.getLogicalLink() == EngineSearchConst.AND){
          res = res && item.isCriterionValid(m);
      }
      else if( item.getLogicalLink() == EngineSearchConst.OR){
        res = res || item.isCriterionValid(m);
      }
      else{
        throw new SearchEngineException("evaluateCriterion : bad logical link");
      }

    }
    return res;
  }

}
