package fr.unice.javinspector.core.loader;
import java.io.*;
import java.net.*;
import java.util.*;
import fr.unice.javinspector.plugin.*;


/**
 * <p>Title: JavInspector</p>
 * <p>Description: a plugin ClassLoader that loads classes <code>fr.unice.javinspector.MVC.Plugin</code>
 * from a directory</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Jerme GAHIDE - Lucas CHARBIT - Gabriel ZERBIB - Xavier GALBOIS
 * @version 1.0
 */


public class PluginLoader {
  private String rep;
  private String packagename;

  /**
   *
   * @param rep Directory where to find the Classes
   * @param packagename optional <code>null</code> or the package name
   */
  public PluginLoader(String rep, String packagename) {
    this.rep = rep;
    this.packagename = packagename;
  }

  /**
   * changes this <code>PluginClassLoader</code> parameters into those supplied
   * @param rep Directory where to find the Classes
   * @param packagename optional <code>null</code> or the package name
   */
  public void setParam(String rep, String packagename) {
    this.rep = rep;
    this.packagename = packagename;
  }

  /**
   * load all the classes in the directory and return them in an Hashtable
   * @return <code>Class</code> of the puglins
   * @throws FileNotFoundException - if the directory does not exists
   */
  public Hashtable loadPlugins() throws FileNotFoundException{
    System.out.println("rep = " + rep );

    return loadClassFromDir(null, rep, packagename);
  }

  /**
   *
   * @param url
   * @param rep
   * @param packagename
   * @return
   * @throws FileNotFoundException
   */
  private Hashtable loadClassFromDir(URL url, String rep, String packagename) throws FileNotFoundException{
    GeneralClassLoader mcl = new FileClassLoader(rep);
    Hashtable plugins = new Hashtable();
    File dir = new File(rep);
    Class maclasse = null;
    Plugin leplugin = null;

    if (dir == null || !dir.isDirectory()) {
      System.err.println("Erreur : " + rep);
      System.err.println(": il ne s'agit pas d'un repertoire");
      throw new FileNotFoundException();
    }
    String[] list = dir.list();
    // Par ordre alphabetique
    Arrays.sort(list);
    for (int i = 0; i < list.length; i++) {
      if (list[i].indexOf('$') == -1 && list[i].endsWith(".class")) {
        // Maintenant on colle le nom du paquetage.
        String nomClasse = /*packagename + "." +*/
          list[i].substring(0,list[i].indexOf('.'));
        //mcl.setClassNameReplacementChar('_');
        try {
          maclasse = mcl.loadClass(nomClasse);
        } catch(Exception ex) {
          System.err.println("le chargement de " + nomClasse + "a echou");
          System.err.println(" pertir de : " + dir);
          ex.printStackTrace();
          continue;
        }
        plugins.put(maclasse.getName(), maclasse);
      }
    }
    return plugins;
  }

  /**
   * Try to instantiate the class with the default constructor
   * @param nomClasse the class to instantiate
   * @return the instantiated class
   */
  private Plugin createInstance(String nomClasse) {
    System.out.println("nom de la classe : " + nomClasse);
    Plugin p1 = null;
    Class c1 = null;

    try {
      c1 = Class.forName(nomClasse);
      p1 = (Plugin) c1.newInstance();
    }
    // on catch bien les erreur
    catch (Exception ex) {
      System.out.println(ex);
    }

    return p1;
  }
}

