package fr.unice.javinspector.core.search;

import java.lang.reflect.*;
import java.util.ArrayList;
import fr.unice.javinspector.core.analyser.ClassifierAnalyser;


/**
 * <p>Title: BasicsSearch</p>
 * <p>Description: This class allows to make a simple search based on an
 * AdvancedAnalyser. You've got some filter to make your search more precise.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: UNICE</p>
 * @author Zerbib Gabriel, Charbit Lucas, Galbois Xavier Gahide Jrme
 * @version 0.1
 */

public class BasicsSearch implements SearchServices{

  /**
   * make a new basic Search object.
   */
  public BasicsSearch(){
  }


  /**
   * Search all the method of the analyser that exactely matches with the pattern.
   * @param analyser the analyser to search into
   * @param name the name of the method search
   * @return the ArrayList of the method that match for this Analyser
   */
  public ArrayList getMethod(ClassifierAnalyser analyser, String name){
    ArrayList result = new ArrayList();
    Method[] m = analyser.getClassAnalysed().getDeclaredMethods();

    for(int i=0 ; i < m.length ; i++){
      if( m[i].getName().compareTo(name) == 0)
        result.add(m[i]);
    }

    return result;
  }


  /**
    * Search the method that match exactely the name and the mod specified.
    * the attr public = 1 private = 2 protected = 3 don't mind = 4
    * is_static and final yes = 1 no = 2 don't mind = 3
    * the type abstract = 1 interface = 2 don't mind = 3
    * @param analyser the analyser to search into
    * @param name the name of the method search
    * @param attr the type of attribute for the method [private, public, protected]
    * @param is_static set the search for a static method or not
    * @param is_final  set the search for a final method or not
    * @param type set if the method is abstract or not.
    * @return the ArrayList of the methods that match for this Analyser
   */
  public ArrayList getMethod(ClassifierAnalyser analyser, String name,
                             int attr,
                             int is_static,
                             int is_final,
                             int type
                             ){

    ArrayList result = new ArrayList();
    Method[] m = analyser.getClassAnalysed().getDeclaredMethods();

    for(int i=0 ; i < m.length ; i++){
      if( m[i].getName().compareTo(name) == 0 &&
          testCondition(m[i], attr, is_static, is_final, type)){
        result.add(m[i]);
        System.out.println("ok "+m[i].getName());
      }
    }
    return result;
  }

  /**
    * Search the method that match exactely the name and the mod specified.
    * the attr public = 1 private = 2 protected = 3 don't mind = 4
    * is_static and final yes = 1 no = 2 don't mind = 3
    * the type abstract = 1 interface = 2 don't mind = 3
    * @param analyser the analyser to search into
    * @param attr the type of attribute for the method [private, public, protected]
    * @param is_static set the search for a static method or not
    * @param is_final  set the search for a final method or not
    * @param type set if the method is abstract or not.
    * @return the ArrayList of the method that match for this Analyser
   */
  public ArrayList getMethod(ClassifierAnalyser analyser,
                                int attr, int is_static,
                                int is_final, int type
                                ){

    ArrayList result = new ArrayList();
    Method[] m = analyser.getClassAnalysed().getDeclaredMethods();

    for(int i=0 ; i < m.length ; i++){
      if( testCondition(m[i], attr, is_static, is_final, type) ){
        result.add(m[i]);
        System.out.println("ok "+m[i].getName());
      }
    }
    return result;
  }


  /**
   *
   * @param m the method to check
   * @param attr the type of attribute for the method [private, public, protected]
   * @param is_static set the search for a static method or not
   * @param is_final set the search for a final method or not
   * @param type set if the method is abstract or not.
   * @return true if the method match with the conditions passed
   * in the parameters of the function.
   */
  private boolean testCondition(Member m,   int attr,
                                int is_static,
                                int is_final,
                                int type
                                ){

    int protection;
    boolean localRes;
    boolean test1 = false;
    boolean test2 = false;
    boolean test3 = false;
    boolean test4 = false;

    // test for the private public or protected attribute
    if(Modifier.isPrivate(m.getModifiers())) {test1 = (2 == attr);}
    if(Modifier.isPublic(m.getModifiers())) {test1 = (1 == attr);}
    if(Modifier.isProtected(m.getModifiers()))  {test1 = (3 == attr);}
    if(attr == 4) {test1 = true;}

    if(test1){

      if(is_static == 3) {test2 = true;}
      if(is_static == 1 && Modifier.isStatic(m.getModifiers())) {test2 = true;}
      if(is_static == 2 && !Modifier.isStatic(m.getModifiers())) {test2 = true;}

      if(is_final == 3) {test3 = true;}
      if(is_final == 1 && Modifier.isFinal(m.getModifiers())) {test3 = true;}
      if(is_final == 2 && !Modifier.isFinal(m.getModifiers())) {test3 = true;}

      if( test2 && test3 )
          if(Modifier.isAbstract(m.getModifiers()) ) {test4 = (1 == type);}
          else {test4 = (3 == type);}

      if(test4){return true;}

    }
    return false;
  }

  /**
   * Search all the variable of the analyser that matches with the pattern name.
   * @param analyser the analyser to search into
   * @param name the name of the variable search
   * @return the ArrayList of the variable that match for this Analyser
   */
  public ArrayList getVariable(ClassifierAnalyser analyser, String name){

    Field[] field_tab = analyser.getClassAnalysed().getDeclaredFields();
    ArrayList result = new ArrayList();

    for(int i=0 ; i < field_tab.length ; i++){
      if( field_tab[i].getName().compareTo(name) == 0)
        result.add(field_tab[i]);
    }

    return result;
  }

  /**
   * Search all the variable of the analyser that matches with the pattern and
   * the condition specified by is_static and is_final.Note that we can't specifie
   * the [public private or protected condition}]. That's normal cause all the value
   * that we are able to get are public.
   * @param analyser the analyser to search into
   * @param name the name of the variable search
   * @param is_static set the search for a static method or not
   * @param is_final  set the search for a final method or not
   * @return the ArrayList of the variable that match for this Analyser
    */
  public ArrayList getVariable(ClassifierAnalyser analyser, String name,
                               int is_static,
                               int is_final
                               ){

    Field[] field_tab = analyser.getClassAnalysed().getDeclaredFields();
    ArrayList result = new ArrayList();

    for(int i=0 ; i < field_tab.length ; i++){
      if( field_tab[i].getName().compareTo(name) == 0 &&
          testCondition(field_tab[i], 4 , is_static, is_final, 3) ){
        result.add(field_tab[i]);
        System.out.println("ok "+field_tab[i].getName());
      }
    }
    return result;
  }

  /**
   * Search all the variable of the analyser that matches with the pattern and
   * the condition specified by is_static and is_final. Note that we can't specifie
   * the [public private or protected condition}]. That's normal cause all the value
   * that we are able to get are public.
   * @param analyser the analyser to search into
   * @param is_static set the search for a static method or not
   * @param is_final  set the search for a final method or not
   */
  public ArrayList getVariable(ClassifierAnalyser analyser,
                               int is_static,
                               int is_final
                               ){

    Field[] field_tab = analyser.getClassAnalysed().getDeclaredFields();
    ArrayList result = new ArrayList();

    for(int i=0 ; i < field_tab.length ; i++){
      if( testCondition(field_tab[i], 4 , is_static, is_final, 3) ){
        result.add(field_tab[i]);
        System.out.println("ok "+field_tab[i].getName());
      }
    }
    return result;


  }



}
