package fr.unice.javinspector.core.search;

/**
 * <p>Title: SearchException</p>
 * <p>Description: The exceptions that can be throw by a Search.
 * All the type of error that are describe here is non allowed pair of keyword.
 * </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: UNICE</p>
 * @author Gahide Jrme - Zerbib Gabriel - Galbois Xavier - Charbit Lucas
 * @version 1.0
 */

public class SearchException extends Exception {

  private static String[] messageRef = new String[2000];
  private static boolean isInit = false;

  /**
   * type error where search is submit with variable and abstract
   */
  protected static final int VARIABLE_ASBTRACT = 0;

  /**
   * type error where search is submit with variable and interface
   */
  protected static final int VARIABLE_INTERFACE = 1;

  /**
   * type error where search is submit with final and abstract
   */
  protected static final int FINAL_ASBTRACT = 3;

  /**
   * type error where search is submit with variable and interface
   */
  protected static final int METHOD_INTERFACE = 4;

  /**
   * type error when the a null pointer occur at the creation of the
   * search result.
   */
  public static final int INTERNAL_ERROR_NULL_POINTER = 1000;

  /**
   * type error occur when the actionTypeSearch is Undefine.
   */
  public static final int UNDEFINE_TYPE_SEARCH = 1010;


  private String message;

  /**
   * Construct a new SearchExecption with a specified message.
   * This Constructor exist in case of the type of error given are
   * not enought
   * @param message the error message.
   */
  public SearchException(String message){
    this.message = message;
  }

  /**
   * Construct a new SearchExecption with a specified message, with a
   * specified type of error.
   * NB : If a bad typeException is given the comportement is not specified.
   * @param typeException the type of error that had occur.
   */
  public SearchException(int typeException){
    if(!isInit)
      InitString();
    this.message = messageRef[typeException];
  }

  /**
   * @return the error message associated to the error.
   * The error message is spefified by the type of the error.
   */
  public String getMessage(){
    return message;
  }


  private void InitString(){
    isInit = true;

    messageRef[VARIABLE_ASBTRACT]  = "can not have a variable abstract";
    messageRef[VARIABLE_INTERFACE] = "can not have a variable interface";
    messageRef[FINAL_ASBTRACT]     = "can not combine keyword final "+
                                     "and abstract the JLS forbit it";
    messageRef[METHOD_INTERFACE]   = "A method can not be an interface";
    messageRef[INTERNAL_ERROR_NULL_POINTER] =
        "PRG Programming error : A null pointer exception has been detected : aborting";
    messageRef[UNDEFINE_TYPE_SEARCH] = "the type of search(method, var ...) is undefined";

  }


}
