package fr.unice.javinspector.core.search;

import fr.unice.javinspector.core.analyser.ClassifierAnalyser;
import java.util.ArrayList;
import java.lang.reflect.Modifier;

/**
 * <p>Title: SearchServices</p>
 * <p>Description: This interface describe the services available by the package
 * Search.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: UNICE</p>
 * @author Zerbib Gabriel, Charbit Lucas, Galbois Xavier Gahide Jrme
 * @version 0.1
 */

public interface SearchServices {

  /**
   * Search all the method of the analyser that matches with the pattern.
   * @param analyser the analyser to search into
   * @param name the name of the method search
   * @return the ArrayList of the method that match for this Analyser

   */
  public ArrayList getMethod(ClassifierAnalyser analyser, String name);

  /**
   * Search the method that match the name and the mod .
   * the attr public = 1 private = 2 protected = 3 don't mind = 4
   * is_static and final yes = 1 no = 2 don't mind = 3
   * the type abstract = 1 interface = 2 don't mind = 3
   * @param analyser the analyser to search into
   * @param name the name of the method search
   * @param attr the type of attribute for the method [private, public, protected]
   * @param is_static set the search for a static method or not
   * @param is_final  set the search for a final method or not
   * @param type set if the method is abstract or not.
   * @return the ArrayList of the method that match for this Analyser
   */
  public ArrayList getMethod(ClassifierAnalyser analyser, String name,
                             int attr,
                             int is_static,
                             int is_final,
                             int type
                             );


  /**
   * Search the method that match the name and the mod .
   * the attr public = 1 private = 2 protected = 3 don't mind = 4
   * is_static and final yes = 1 no = 2 don't mind = 3
   * the type abstract = 1 interface = 2 don't mind = 3
   * @param analyser the analyser to search into
   * @param attr the type of attribute for the method [private, public, protected]
   * @param is_static set the search for a static method or not
   * @param is_final  set the search for a final method or not
   * @param type set if the method is abstract or not.
   * @return the ArrayList of the method that match for this Analyser
   */
  public ArrayList getMethod(ClassifierAnalyser analyser,
                               int attr,
                               int is_static,
                               int is_final,
                               int type
                             );


  /**
   * Search all the variable of the analyser that matches with the pattern.
   * @param analyser the analyser to search into
   * @param name the name of the variable search
   * @return the ArrayList of the variable that match for this Analyser

   */
  public ArrayList getVariable(ClassifierAnalyser analyser, String name);

  /**
   * Search all the variable of the analyser that matches with the pattern and
   * the condition specified by is_static and is_final. Note that we can't specifie
   * the [public private or protected condition}]. That's normal cause all the value
   * that we are able to get are public.
   * @param analyser the analyser to search into
   * @param name the name of the variable search
   * @param is_static set the search for a static method or not
   * @param is_final  set the search for a final method or not
   * @return the ArrayList of the variable that match for this Analyser

   */
  public ArrayList getVariable(ClassifierAnalyser analyser, String name,
                               int is_static,
                               int is_final
                               );

  /**
   * Search all the variable of the analyser that matches with the pattern and
   * the condition specified by is_static and is_final. Note that we can't specifie
   * the [public private or protected condition}]. That's normal cause all the value
   * that we are able to get are public.
   * @param analyser the analyser to search into
   * @param is_static set the search for a static method or not
   * @param is_final  set the search for a final method or not

   */
  public ArrayList getVariable(ClassifierAnalyser analyser,
                               int is_static,
                               int is_final
                               );





}
